@@ -1,18 +1,43 @@
+# Copyright (C) 2022 - 2025 ANSYS, Inc. and/or its affiliates.
+# SPDX-License-Identifier: MIT
+#
+#
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be included in all
+# copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
+
 """Contains Optislang class, which provides the Python API for the optiSLang app."""
 from __future__ import annotations
 
+from importlib.metadata import version
+from logging import Logger
 from pathlib import Path
 from typing import TYPE_CHECKING, Iterable, Mapping, Optional, Sequence, Tuple, Union
 
-from importlib_metadata import version
+from deprecated.sphinx import deprecated
 
 from ansys.optislang.core import LOG
-from ansys.optislang.core.project import Project
-from ansys.optislang.core.tcp_osl_server import TcpOslServer
+from ansys.optislang.core.tcp.application import TcpApplicationProxy
+from ansys.optislang.core.tcp.osl_server import TcpOslServer
 
 if TYPE_CHECKING:
-    from ansys.optislang.core.logging import OslLogger
-    from ansys.optislang.core.osl_server import OslServer
+    from ansys.optislang.core.application import Application
+    from ansys.optislang.core.osl_server import OslServer, OslVersion
+    from ansys.optislang.core.project import Project
 
 
 class Optislang:
@@ -29,16 +54,16 @@ class Optislang:
 
     Parameters
     ----------
-    host : str, optional
+    host : Optional[str], optional
         IPv4/v6 address or domain name on which optiSLang is running as a
         server. The default is ``None``.
-    port : int, optional
+    port : Optional[int], optional
         Port on which optiSLang is running as a server. The default is ``None``.
-    executable : Union[str, pathlib.Path], optional
+    executable : Optional[Union[str, pathlib.Path]], optional
         Path to the optiSLang executable file to execute on a the local host.
         The default is ``None``. This parameter is ignored when ``host``
         and ``port`` parameters are specified.
-    project_path : Union[str, pathlib.Path], optional
+    project_path : Optional[Union[str, pathlib.Path]], optional
         Path to the optiSLang project file that a new local optiSLang server
         is to use. The default is ``None``. This parameter is ignored
         when ``host`` and ``port`` parameters are specified. Here is how
@@ -50,9 +75,24 @@ class Optislang:
 
     batch : bool, optional
         Determines whether to start optiSLang server in batch mode. Defaults to ``True``.
-    port_range : Tuple[int, int], optional
-        Defines the port range for optiSLang server. Defaults to ``None``.
-    no_run : bool, optional
+
+        ..note:: Cannot be used in combination with service mode.
+
+    service: bool, optional
+        Determines whether to start optiSLang server in service mode. If ``True``,
+        ``batch`` argument is set to ``False``. Defaults to ``False``.
+
+        ..note:: Cannot be used in combination with batch mode.
+
+    server_address : Optional[str], optional
+        In case an optiSLang server is to be started, this defines the address
+        of the optiSLang server. If not specified, optiSLang will be listening on
+        local host only. Defaults to ``None``.
+    port_range : Optional[Tuple[int, int]], optional
+        In case an optiSLang server is to be started, this restricts the port range
+        for the optiSLang server. If not specified, optiSLang will be allowed to
+        listen on any port. Defaults to ``None``.
+    no_run : Optional[bool], optional
         Determines whether not to run the specified project when started in batch mode.
         Defaults to ``None``.
 
@@ -84,20 +124,26 @@ class Optislang:
 
         .. note:: Only supported in batch mode.
 
-    listener_id : str, optional
+    listener_id : Optional[str], optional
         Specific unique ID for the TCP listener. Defaults to ``None``.
     multi_listener : Iterable[Tuple[str, int, Optional[str]]], optional
         Multiple remote listeners (plain TCP/IP based) to be registered at optiSLang server.
         Each listener is a combination of host, port and (optionally) listener ID.
         Defaults to ``None``.
+    listeners_refresh_interval : int, optional
+        Refresh interval for TCP listeners in seconds. Defaults to 10 s.
+    listeners_default_timeout : Optional[int], optional
+        Default timeout for TCP listeners in milliseconds. Defaults to ``None`` which results in
+        optiSLang using the default timeout value of 60000 milliseconds.
     ini_timeout : float, optional
-        Time in seconds to connect to the optiSLang server. The default is ``20``.
-    name : str, optional
-        ID of the optiSLang instance.
-    password : str, optional
+        Time in seconds to connect to the optiSLang server. The default is ``60``.
+    name : Optional[str], optional
+        ID of the optiSLang instance. Defaults to ``None``.
+    password : Optional[str], optional
         Server password. The default is ``None``. This parameter is used when
         communication with the server requires that the request contain a password.
-    loglevel : str, optional
+        Defaults to ``None``.
+    loglevel : Optional[str], optional
         Logging level. The options are:
 
         - ``CRITICAL``: Log errors that are fatal for the app.
@@ -106,6 +152,12 @@ class Optislang:
         - ``INFO``: Log some useful information that the program works as expected.
         - ``DEBUG``: Log all information for use in debugging.
 
+        Defaults to ``None``.
+
+    log_process_stdout : bool, optional
+        Determines whether the process stdout should be logged. Defaults to ``False``.
+    log_process_stderr : bool, optional
+        Determines whether the process stderr should be logged. Defaults to ``False``.
     shutdown_on_finished: bool, optional
         Whether to shut down when execution is finished and no listeners are registered.
         The default is ``True``. This parameter is ignored when ``host`` and
@@ -113,40 +165,40 @@ class Optislang:
 
         .. note:: Only supported in batch mode.
 
-    env_vars : Mapping[str, str], optional
+    env_vars : Optional[Mapping[str, str]], optional
         Additional environmental variables (key and value) for the optiSLang server process.
         Defaults to ``None``.
-    import_project_properties_file : Union[str, pathlib.Path], optional
+    import_project_properties_file : Optional[Union[str, pathlib.Path]], optional
         Optional path to a project properties file to import. Defaults to ``None``.
-    export_project_properties_file : Union[str, pathlib.Path], optional
+    export_project_properties_file : Optional[Union[str, pathlib.Path]], optional
         Optional path to a project properties file to export. Defaults to ``None``.
 
         .. note:: Only supported in batch mode.
 
-    import_placeholders_file : Union[str, pathlib.Path], optional
+    import_placeholders_file : Optional[Union[str, pathlib.Path]], optional
         Optional path to a placeholders file to import. Defaults to ``None``.
-    export_placeholders_file : Union[str, pathlib.Path], optional
+    export_placeholders_file : Optional[Union[str, pathlib.Path]], optional
         Optional path to a placeholders file to export. Defaults to ``None``.
 
         .. note:: Only supported in batch mode.
 
-    output_file : Union[str, pathlib.Path], optional
+    output_file : Optional[Union[str, pathlib.Path]], optional
         Optional path to an output file for writing project run results to. Defaults to ``None``.
 
         .. note:: Only supported in batch mode.
 
-    dump_project_state : Union[str, pathlib.Path], optional
+    dump_project_state : Optional[Union[str, pathlib.Path]], optional
         Optional path to a project state dump file to export. If a relative path is provided,
         it is considered to be relative to the project working directory. Defaults to ``None``.
 
         .. note:: Only supported in batch mode.
 
-    opx_project_definition_file : Union[str, pathlib.Path], optional
+    opx_project_definition_file : Optional[Union[str, pathlib.Path]], optional
         Optional path to an OPX project definition file. Defaults to ``None``.
 
         .. note:: Only supported in batch mode.
 
-    additional_args : Iterable[str], optional
+    additional_args : Optional[Iterable[str]], optional
         Additional command line arguments used for execution of the optiSLang server process.
         Defaults to ``None``.
 
@@ -155,6 +207,10 @@ class Optislang:
     RuntimeError
         Raised when the connection to the optiSLang server cannot be established
         before the specified timeout.
+    OslServerStartError
+        Raised when optiSLang server process failed to start
+    OslServerLicensingError
+        Raised when optiSLang server process failed to start due to licensing issues
 
     Examples
     --------
@@ -163,40 +219,46 @@ class Optislang:
 
     >>> from ansys.optislang.core import Optislang
     >>> osl = Optislang()
-    >>> osl_version = osl.get_osl_version_string()
+    >>> osl_version = osl.osl_version_string
     >>> print(osl_version)
     >>> osl.dispose()
     """
 
     def __init__(
         self,
-        host: str = None,
-        port: int = None,
-        executable: Union[str, Path] = None,
-        project_path: Union[str, Path] = None,
+        host: Optional[str] = None,
+        port: Optional[int] = None,
+        executable: Optional[Union[str, Path]] = None,
+        project_path: Optional[Union[str, Path]] = None,
         batch: bool = True,
-        port_range: Tuple[int, int] = None,
-        no_run: bool = None,
+        service: bool = False,
+        server_address: Optional[str] = None,
+        port_range: Optional[Tuple[int, int]] = None,
+        no_run: Optional[bool] = None,
         no_save: bool = False,
         force: bool = True,
         reset: bool = False,
         auto_relocate: bool = False,
-        listener_id: str = None,
-        multi_listener: Iterable[Tuple[str, int, Optional[str]]] = None,
-        ini_timeout: Union[int, float] = 20,
-        name: str = None,
-        password: str = None,
-        loglevel: str = None,
+        listener_id: Optional[str] = None,
+        multi_listener: Optional[Iterable[Tuple[str, int, Optional[str]]]] = None,
+        listeners_refresh_interval: int = 10,
+        listeners_default_timeout: Optional[int] = None,
+        ini_timeout: Union[int, float] = 60,
+        name: Optional[str] = None,
+        password: Optional[str] = None,
+        loglevel: Optional[str] = None,
+        log_process_stdout: bool = False,
+        log_process_stderr: bool = False,
         shutdown_on_finished: bool = True,
-        env_vars: Mapping[str, str] = None,
-        import_project_properties_file: Union[str, Path] = None,
-        export_project_properties_file: Union[str, Path] = None,
-        import_placeholders_file: Union[str, Path] = None,
-        export_placeholders_file: Union[str, Path] = None,
-        output_file: Union[str, Path] = None,
-        dump_project_state: Union[str, Path] = None,
-        opx_project_definition_file: Union[str, Path] = None,
-        additional_args: Iterable[str] = None,
+        env_vars: Optional[Mapping[str, str]] = None,
+        import_project_properties_file: Optional[Union[str, Path]] = None,
+        export_project_properties_file: Optional[Union[str, Path]] = None,
+        import_placeholders_file: Optional[Union[str, Path]] = None,
+        export_placeholders_file: Optional[Union[str, Path]] = None,
+        output_file: Optional[Union[str, Path]] = None,
+        dump_project_state: Optional[Union[str, Path]] = None,
+        opx_project_definition_file: Optional[Union[str, Path]] = None,
+        additional_args: Optional[Iterable[str]] = None,
     ) -> None:
         """Initialize a new instance of the ``Optislang`` class."""
         self.__host = host
@@ -204,6 +266,8 @@ class Optislang:
         self.__executable = Path(executable) if executable is not None else None
         self.__project_path = Path(project_path) if project_path is not None else None
         self.__batch = batch
+        self.__service = service
+        self.__server_address = server_address
         self.__port_range = port_range
         self.__no_run = no_run
         self.__no_save = no_save
@@ -217,6 +281,8 @@ class Optislang:
         self.__env_vars = env_vars
         self.__listener_id = listener_id
         self.__multi_listener = multi_listener
+        self.__listeners_refresh_interval = listeners_refresh_interval
+        self.__listeners_default_timeout = listeners_default_timeout
         self.__import_project_properties_file = import_project_properties_file
         self.__export_project_properties_file = export_project_properties_file
         self.__import_placeholders_file = import_placeholders_file
@@ -226,11 +292,14 @@ class Optislang:
         self.__opx_project_definition_file = opx_project_definition_file
         self.__additional_args = additional_args
         self.__logger = LOG.add_instance_logger(self.name, self, loglevel)
+        self.__log_process_stdout = log_process_stdout
+        self.__log_process_stderr = log_process_stderr
         self.__osl_server: OslServer = self.__init_osl_server("tcp")
-        project_uid = self.__osl_server.get_project_uid()
-        self.__project = (
-            Project(osl_server=self.__osl_server, uid=project_uid) if project_uid else None
-        )
+        self.__application: Application = self.__init_application()
+
+        if self.__project_path and not self.__batch and not self.__service:  # pragma: no cover
+            # trigger lazy project load in GUI mode
+            self.open(file_path=self.__project_path, force=self.__force, reset=self.__reset)
 
     def __init_osl_server(self, server_type: str) -> OslServer:
         """Initialize optiSLang server.
@@ -250,19 +319,27 @@ class Optislang:
         ------
         NotImplementedError
             Raised when the specified server type is not supported.
+        OslServerStartError
+            Raised when optiSLang server process failed to start
+        OslServerLicensingError
+            Raised when optiSLang server process failed to start due to licensing issues
         """
         if server_type.lower() == "tcp":
             return TcpOslServer(
                 host=self.__host,
                 port=self.__port,
                 executable=self.__executable,
-                project_path=self.__project_path,
+                project_path=self.__project_path if self.__batch else None,
                 no_save=self.__no_save,
                 ini_timeout=self.__ini_timeout,
                 password=self.__password,
                 logger=self.log,
+                log_process_stdout=self.__log_process_stdout,
+                log_process_stderr=self.__log_process_stderr,
                 shutdown_on_finished=self.__shutdown_on_finished,
                 batch=self.__batch,
+                service=self.__service,
+                server_address=self.__server_address,
                 port_range=self.__port_range,
                 no_run=self.__no_run,
                 force=self.__force,
@@ -278,16 +355,26 @@ class Optislang:
                 opx_project_definition_file=self.__opx_project_definition_file,
                 listener_id=self.__listener_id,
                 multi_listener=self.__multi_listener,
+                listeners_refresh_interval=self.__listeners_refresh_interval,
+                listeners_default_timeout=self.__listeners_default_timeout,
                 additional_args=self.__additional_args,
             )
         else:
-            raise NotImplementedError(f'OptiSLang server of type "{server_type}" is not supported.')
+            raise NotImplementedError(
+                f"OptiSLang server of type ``{server_type}`` is not supported."
+            )
+
+    def __init_application(self) -> Application:
+        if isinstance(self.__osl_server, TcpOslServer):
+            return TcpApplicationProxy(osl_server=self.__osl_server, logger=self.log)
+        else:
+            raise NotImplementedError("Currently supported only for ``TcpOslServer``.")
 
     def __str__(self):
         """Return product name, version of optiSLang, and version of PyOptiSLang."""
         return (
             f"Product name: optiSLang\n"
-            f"Version: {self.get_osl_version_string()}\n"
+            f"Version: {self.osl_version_string}\n"
             f"PyOptiSLang: {version('ansys.optislang.core')}"
         )
 
@@ -314,21 +401,18 @@ class Optislang:
         self.dispose()
 
     @property
-    def name(self) -> str:
-        """Unique ID of the optiSLang instance."""
-        if not self.__name:
-            if self.__host or self.__port:
-                self.__name = f"optiSLang_{self.__host}:{self.__port}"
-            else:
-                self.__name = f"optiSLang_{id(self)}"
-        return self.__name
+    def application(self) -> Application:
+        """Instance of the ``Application`` class.
 
-    @property
-    def log(self) -> OslLogger:
-        """Instance logger."""
-        return self.__logger
+        Returns
+        -------
+        Application
+            Instance of ``Application`` class for operating with project.
+        """
+        return self.__application
 
     @property
+    @deprecated(version="0.6.0", reason="Use `Optislang.application.project is not None` instead.")
     def has_active_project(self) -> bool:
         """
         Whether a project is loaded.
@@ -347,34 +431,107 @@ class Optislang:
         TimeoutError
             Raised when the timeout float value expires.
         """
-        return self.__osl_server.get_project_uid() is not None
+        return self.application.project is not None
 
     @property
-    def project(self) -> Project:
+    def log(self) -> Logger:
+        """Instance logger."""
+        return self.__logger
+
+    @property
+    def name(self) -> str:
+        """Unique ID of the optiSLang instance."""
+        if not self.__name:
+            if self.__host or self.__port:
+                self.__name = f"optiSLang_{self.__host}:{self.__port}"
+            else:
+                self.__name = f"optiSLang_{id(self)}"
+        return self.__name
+
+    @property
+    def osl_server(self) -> OslServer:
+        """Get the currently used instance of the OslServer.
+
+        This instance can be used to directly communicate with optiSLang using
+        the optiSLang server API.
+
+        Returns
+        -------
+        Optional[OslServer]
+            OptiSLang server object.
+        """
+        return self.__osl_server
+
+    @property
+    def osl_version(self) -> OslVersion:
+        """Version of used optiSLang.
+
+        Returns
+        -------
+        OslVersion
+            optiSLang version as typing.NamedTuple containing
+            major, minor, maintenance and revision versions.
+        """
+        return self.__osl_server.osl_version
+
+    @property
+    def osl_version_string(self) -> str:
+        """Version of used optiSLang.
+
+        Returns
+        -------
+        str
+            optiSLang version.
+        """
+        return self.__osl_server.osl_version_string
+
+    @property
+    def project(self) -> Optional[Project]:
         """Instance of the ``Project`` class.
 
         Returns
         -------
-        Project
+        Optional[Project]
             Loaded project. If no project is loaded, ``None`` is returned.
         """
-        return self.__project
+        return self.application.project
 
-    # close method doesn't work properly in optiSLang 2023 R1, Thus, it was commented out
-    # def close(self) -> None:
-    #     """Close the current project.
+    @property
+    def timeout(self) -> Optional[float]:
+        """Get the timeout value for executing commands.
 
-    #     Raises
-    #     ------
-    #     OslCommunicationError
-    #         Raised when an error occurs while communicating with the server.
-    #     OslCommandError
-    #         Raised when a command or query fails.
-    #     TimeoutError
-    #         Raised when the timeout float value expires.
-    #     """
-    #     self.__osl_server.close()
-    #     self.__project = None
+        Returns
+        -------
+        timeout: Optional[float]
+            Timeout in seconds to perform commands. This value must be greater
+            than zero or ``None``. The default is ``None``. Another function
+            raises a timeout exception if the timeout value has elapsed before
+            an operation has completed. If the timeout is ``None``, functions
+            wait until they're finished, and no timeout exception is raised.
+        """
+        return self.__osl_server.timeout
+
+    @timeout.setter
+    def timeout(self, timeout: Optional[float] = None) -> None:
+        """Set the timeout value for the executing commands.
+
+        Parameters
+        ----------
+        timeout: Optional[float], optional
+            Timeout in seconds to perform commands. This value must be greater
+            than zero or ``None``. The default is ``None``. Another function
+            raises a timeout exception if the timeout value has elapsed before
+            an operation has completed. If the timeout is ``None``, functions
+            wait until they're finished, and no timeout exception is raised.
+
+        Raises
+        ------
+        ValueError
+            Raised when the timeout value is less than or equal to 0.
+        TypeError
+            Raised when the timeout is not a type of ``float`` or  ``None``.
+        """
+        self.__osl_server.timeout = timeout
 
     def dispose(self) -> None:
         """Close all local threads and unregister listeners.
@@ -390,6 +547,21 @@ class Optislang:
         """
         self.__osl_server.dispose()
 
+    @deprecated(version="0.6.0", reason="Use :py:attr:`Optislang.osl_server` instead.")
+    def get_osl_server(self) -> Optional[OslServer]:
+        """Get the currently used instance of the OslServer.
+
+        This instance can be used to directly communicate with optiSLang using
+        the optiSLang server API.
+
+        Returns
+        -------
+        Optional[OslServer]
+            OptiSLang server object.
+        """
+        return self.osl_server
+
+    @deprecated(version="0.5.0", reason="Use :py:attr:`Optislang.osl_version_string` instead.")
     def get_osl_version_string(self) -> str:
         """Get the optiSLang version in use as a string.
 
@@ -407,16 +579,17 @@ class Optislang:
         TimeoutError
             Raised when the timeout float value expires.
         """
-        return self.__osl_server.get_osl_version_string()
+        return self.__osl_server.osl_version_string
 
-    def get_osl_version(self) -> Tuple[Union[int, None], ...]:
+    @deprecated(version="0.5.0", reason="Use :py:attr:`Optislang.osl_version` instead.")
+    def get_osl_version(self) -> OslVersion:
         """Get the optiSLang version in use as a tuple.
 
         Returns
         -------
-        tuple
-            optiSLang version as tuple contains the
-            major version, minor version, maintenance version, and revision.
+        OslVersion
+            optiSLang version as typing.NamedTuple containing
+            major, minor, maintenance and revision versions.
 
         Raises
         ------
@@ -424,40 +597,41 @@ class Optislang:
             Raised when an error occurs while communicating with the server.
         OslCommandError
             Raised when a command or query fails.
+        RuntimeError
+            Raised when parsing version numbers from string fails.
         TimeoutError
             Raised when the timeout float value expires.
         """
-        return self.__osl_server.get_osl_version()
+        return self.__osl_server.osl_version
 
-    def get_timeout(self) -> Union[float, None]:
+    @deprecated(version="0.5.0", reason="Use :py:attr:`Optislang.timeout` instead.")
+    def get_timeout(self) -> Optional[float]:
         """Get the timeout value for executing commands.
 
         Returns
         -------
-        timeout: Union[float, None]
+        timeout: Optional[float]
             Timeout in seconds to perform commands. This value must be greater
             than zero or ``None``. The default is ``None``. Another function
             raises a timeout exception if the timeout value has elapsed before
             an operation has completed. If the timeout is ``None``, functions
             wait until they're finished, and no timeout exception is raised.
-
-        Raises
-        ------
-        OslCommunicationError
-            Raised when an error occurs while communicating with the server.
-        OslCommandError
-            Raised when a command or query fails.
-        TimeoutError
-            Raised when the timeout float value expires.
         """
-        return self.__osl_server.get_timeout()
+        return self.osl_server.timeout
 
-    def get_working_dir(self) -> Path:
+    @deprecated(
+        version="0.6.0",
+        reason=(
+            "This functionality was moved to "
+            ":py:class:`Project <ansys.optislang.core.project.Project>`."
+        ),
+    )
+    def get_working_dir(self) -> Optional[Path]:
         """Get the path to the optiSLang project's working directory.
 
         Returns
         -------
-        pathlib.Path
+        Optional[pathlib.Path]
             Path to the optiSLang project's working directory. If no project is loaded
             in optiSLang, ``None`` is returned.
 
@@ -470,10 +644,19 @@ class Optislang:
         TimeoutError
             Raised when the timeout float value expires.
         """
-        return self.__osl_server.get_working_dir()
+        if self.application.project is None:
+            return None
+        return self.application.project.get_working_dir()
 
+    @deprecated(
+        version="0.6.0",
+        reason=(
+            "This functionality was moved to "
+            ":py:class:`Application <ansys.optislang.core.application.Application>`."
+        ),
+    )
     def new(self) -> None:
-        """Create a project.
+        """Create and open a new project.
 
         Raises
         ------
@@ -484,17 +667,22 @@ class Optislang:
         TimeoutError
             Raised when the timeout float value expires.
         """
-        self.__osl_server.new()
-        self.__project = Project(
-            osl_server=self.__osl_server, uid=self.__osl_server.get_project_uid()
-        )
+        self.application.new()
 
+    @deprecated(
+        version="0.6.0",
+        reason=(
+            "This functionality was moved to "
+            ":py:class:`Application <ansys.optislang.core.application.Application>`."
+        ),
+    )
     def open(
         self,
         file_path: Union[str, Path],
         force: bool = True,
         restore: bool = False,
         reset: bool = False,
+        project_properties_file: Optional[str] = None,
     ) -> None:
         """Open a project.
 
@@ -513,6 +701,8 @@ class Optislang:
             Whether to restore the project from the last (auto) save point (if present).
         reset : bool, optional
             Whether to reset the project after loading it.
+        project_properties_file : Optional[str], optional
+            Project properties file to import, by default ``None``.
 
         Raises
         ------
@@ -523,11 +713,21 @@ class Optislang:
         TimeoutError
             Raised when the timeout float value expires.
         """
-        self.__osl_server.open(file_path=file_path, force=force, restore=restore, reset=reset)
-        self.__project = Project(
-            osl_server=self.__osl_server, uid=self.__osl_server.get_project_uid()
+        self.application.open(
+            file_path=file_path,
+            force=force,
+            restore=restore,
+            reset=reset,
+            project_properties_file=project_properties_file,
         )
 
+    @deprecated(
+        version="0.6.0",
+        reason=(
+            "This functionality was moved to "
+            ":py:class:`Project <ansys.optislang.core.project.Project>`."
+        ),
+    )
     def reset(self) -> None:
         """Reset the project.
 
@@ -540,8 +740,18 @@ class Optislang:
         TimeoutError
             Raised when the timeout float value expires.
         """
-        self.__osl_server.reset()
+        if self.application.project is not None:
+            self.application.project.reset()
+        else:
+            self.log.error("Cannot reset: No project loaded.")
 
+    @deprecated(
+        version="0.6.0",
+        reason=(
+            "This functionality was moved to "
+            ":py:class:`Project <ansys.optislang.core.project.Project>`."
+        ),
+    )
     def run_python_script(
         self,
         script: str,
@@ -571,8 +781,18 @@ class Optislang:
         TimeoutError
             Raised when the timeout float value expires.
         """
-        return self.__osl_server.run_python_script(script, args)
+        if self.application.project is not None:
+            return self.application.project.run_python_script(script, args)
+        self.log.error("Cannot run Python script: No project loaded.")
+        return ("", "")
 
+    @deprecated(
+        version="0.6.0",
+        reason=(
+            "This functionality was moved to "
+            ":py:class:`Project <ansys.optislang.core.project.Project>`."
+        ),
+    )
     def run_python_file(
         self,
         file_path: Union[str, Path],
@@ -603,8 +823,18 @@ class Optislang:
         TimeoutError
             Raised when the timeout float value expires.
         """
-        return self.__osl_server.run_python_file(file_path, args)
+        if self.application.project is not None:
+            return self.application.project.run_python_file(file_path, args)
+        self.log.error("Cannot run Python file: No project loaded.")
+        return ("", "")
 
+    @deprecated(
+        version="0.6.0",
+        reason=(
+            "This functionality was moved to "
+            ":py:class:`Application <ansys.optislang.core.application.Application>`."
+        ),
+    )
     def save(self) -> None:
         """Save changes to the project data and settings.
 
@@ -617,8 +847,15 @@ class Optislang:
         TimeoutError
             Raised when the timeout float value expires.
         """
-        return self.__osl_server.save()
+        return self.application.save()
 
+    @deprecated(
+        version="0.6.0",
+        reason=(
+            "This functionality was moved to "
+            ":py:class:`Application <ansys.optislang.core.application.Application>`."
+        ),
+    )
     def save_as(
         self,
         file_path: Union[str, Path],
@@ -653,12 +890,19 @@ class Optislang:
         TimeoutError
             Raised when the timeout float value expires.
         """
-        return self.__osl_server.save_as(
+        return self.application.save_as(
             file_path=file_path, force=force, restore=restore, reset=reset
         )
 
+    @deprecated(
+        version="0.6.0",
+        reason=(
+            "This functionality was moved to "
+            ":py:class:`Application <ansys.optislang.core.application.Application>`."
+        ),
+    )
     def save_copy(self, file_path: Union[str, Path]) -> None:
-        """Save a copy of the project to a specified location.
+        """Save a copy of the project to a specified location..
 
         Parameters
         ----------
@@ -674,14 +918,15 @@ class Optislang:
         TimeoutError
             Raised when the timeout float value expires.
         """
-        self.__osl_server.save_copy(file_path)
+        self.application.save_copy(file_path)
 
-    def set_timeout(self, timeout: Union[float, None] = None) -> None:
+    @deprecated(version="0.5.0", reason="Use :py:attr:`Optislang.timeout` instead.")
+    def set_timeout(self, timeout: Optional[float] = None) -> None:
         """Set the timeout value for the executing commands.
 
         Parameters
         ----------
-        timeout: Union[float, None]
+        timeout: Optional[float]
             Timeout in seconds to perform commands. This value must be greater
             than zero or ``None``. The default is ``None``. Another function
             raises a timeout exception if the timeout value has elapsed before
@@ -690,16 +935,12 @@ class Optislang:
 
         Raises
         ------
-        OslCommunicationError
-            Raised when an error occurs while communicating with the server.
-        OslCommandError
-            Raised when a command or query fails.
         ValueError
             Raised when the timeout value is less than or equal to 0.
         TypeError
-            Raised when the timeout is not a Union[float, None].
+            Raised when the timeout is not type of ``float`` or ``None``.
         """
-        self.__osl_server.set_timeout(timeout)
+        self.osl_server.timeout = timeout
 
     def shutdown(self, force: bool = False) -> None:
         """Shut down the optiSLang server.
@@ -712,8 +953,7 @@ class Optislang:
         ----------
         force : bool, optional
             Whether to forcibly shut down a local optiSLang server. The default is
-            ``False``. This parameter has no effect when the connection established
-            is to a remote optiSLang server. In all cases, an attempt is made to
+            ``False``. In all cases, an attempt is made to
             shut down the optiSLang server in the proper way. However, if the
             ``force`` parameter is ``True``, after a while, the process is forcibly
             shut down without an exception being raised.
@@ -728,8 +968,15 @@ class Optislang:
         TimeoutError
             Raised when the ``force`` parameter is ``False`` and the timeout float value expires.
         """
-        self.__osl_server.shutdown(force)
+        self.osl_server.shutdown(force)
 
+    @deprecated(
+        version="0.6.0",
+        reason=(
+            "This functionality was moved to "
+            ":py:class:`Project <ansys.optislang.core.project.Project>`."
+        ),
+    )
     def start(self, wait_for_started: bool = True, wait_for_finished: bool = True) -> None:
         """Start project execution.
 
@@ -758,8 +1005,20 @@ class Optislang:
         TimeoutError
             Raised when the timeout float value expires.
         """
-        self.__osl_server.start(wait_for_started, wait_for_finished)
+        if self.application.project is not None:
+            self.application.project.start(
+                wait_for_started=wait_for_started, wait_for_finished=wait_for_finished
+            )
+        else:
+            self.log.error("Cannot start: No project loaded.")
 
+    @deprecated(
+        version="0.6.0",
+        reason=(
+            "This functionality was moved to "
+            ":py:class:`Project <ansys.optislang.core.project.Project>`."
+        ),
+    )
     def stop(self, wait_for_finished: bool = True) -> None:
         """Stop project execution.
 
@@ -781,36 +1040,7 @@ class Optislang:
         TimeoutError
             Raised when the timeout float value expires.
         """
-        self.__osl_server.stop(wait_for_finished)
-
-    def get_osl_server(self) -> Union[OslServer, None]:
-        """Get the currently used instance of the OslServer.
-
-        This instance can be used to directly communicate with optiSLang using
-        the optiSLang server API.
-        """
-        return self.__osl_server
-
-    # stop_gently method doesn't work properly in optiSLang 2023 R1. Thus, it was commented out
-    # def stop_gently(self, wait_for_finished: bool = True) -> None:
-    #     """Stop project execution after the design is finished.
-
-    #     Parameters
-    #     ----------
-    #     wait_for_finished : bool, optional
-    #         Whether this function call should wait for optiSLang to finish
-    #         the command execution. The default is ``True``, in which case the
-    #         function call does not continue on to the next line of the Python
-    #         script after the command is successfully sent to optiSLang but
-    #         rather waits for execution of the flow inside optiSLang to finish.
-
-    #     Raises
-    #     ------
-    #     OslCommunicationError
-    #         Raised when an error occurs while communicating with the server.
-    #     OslCommandError
-    #         Raised when a command or query fails.
-    #     TimeoutError
-    #         Raised when the timeout float value expires.
-    #     """
-    #     self.__osl_server.stop_gently(wait_for_finished)
+        if self.application.project is not None:
+            self.application.project.stop(wait_for_finished=wait_for_finished)
+        else:
+            self.log.error("Cannot stop: No project loaded.")
